extern crate reqwest;
extern crate serde;
extern crate serde_json;

use dotenv;
use std::collections::HashMap;
use std::str::FromStr;

use serde::{Deserialize, Deserializer};
use serde::de::{self};
use serde::export::fmt::Display;
use serde_json::Value;
use std::env;

#[derive(Deserialize, PartialEq, Debug)]
struct AOCRequest {
    event: String,
    owner_id: String,
    members: HashMap<u32, User>,
}

#[derive(Deserialize, PartialEq, Debug)]
struct User {
    global_score: u32,
    name: Value,
    stars: u8,
    completion_day_level: HashMap<String, HashMap<String, Star>>,
    last_star_ts: Value,
    #[serde(deserialize_with = "from_str")]
    id: u64,
    local_score: u64,
}

#[derive(Deserialize, Eq, PartialEq, Debug)]
struct Star {
    #[serde(deserialize_with = "from_str")]
    get_star_ts: u64
}

fn from_str<'de, T, D>(deserializer: D) -> Result<T, D::Error>
    where T: FromStr,
          T::Err: Display,
          D: Deserializer<'de>
{
    let s = String::deserialize(deserializer)?;
    T::from_str(&s).map_err(de::Error::custom)
}


fn main() {
    dotenv::dotenv().ok();
    let token = env::var_os("WH_TOKEN").expect("Token not set!");
    let id = env::var_os("WH_ID").expect("ID not set!");
    let sess = env::var_os("WH_SESSION").expect("Session not set!");

    let client = reqwest::Client::new();
    let json: AOCRequest = client.get("https://adventofcode.com/2019/leaderboard/private/view/356604.json")
        .header("Cookie",
                format!("session={}", sess.as_os_str().to_str().expect("oof")))
        .send().expect("Failed to send request").json().expect("Error parsing response");

    let id = id.as_os_str().to_str().expect("ID is not an int").parse().expect("ID is not an int");
    let token = token.to_str().expect("Token is not valid");
    let http = serenity::http::Http::default();
    let mut wh = http.as_ref().get_webhook_with_token(id, token).expect("Cannot create webhook");

    wh.edit(&http, Some("AOCHook"), Some(include_str!("./faviconlife"))).expect("oof?");
    let mut res: Vec<&User> = json.members.iter().filter_map(|(_k, v)| {
        if v.name.is_string() {
            return Some(v);
        }
        None
    }).collect();
    res.sort_by(|x, y| x.local_score.cmp(&y.local_score).reverse());

    let k = res.iter().filter_map(|v| v.name.as_str()).max_by_key(|s| s.len()).unwrap_or("").len() + 2;
    let l: Vec<String> = res.iter().enumerate().map(|(i, u)| format!("{:0>2} - {:^len$} - {:0>4}", i + 1, u.name.as_str().unwrap(), u.local_score, len = k)).collect();

    wh.execute(&http, false, |w| {
        w.content(format!("```{}```", l.join("\n")));
        w
    }).expect("oof");
}
